/*
 * MyArrayList.cpp implements the MyArrayList class behavior declared in MyArrayList.h.
 *
 * Today's version is able to resize the array to grow it as needed if it
 * becomes full. The various member functions don't check indexes to see if they are out
 * of bounds.  Also the list can store only ints.  We will address these issues later.
 */

#include <string>
#include "MyArrayList.h"

using namespace std;

// helper that sets all elements of an array to 0
template<typename T>
static void zeroOut(T* a, int length);

template<typename T>
MyArrayList<T>::MyArrayList() {
    zeroOut(m_elements, m_capacity);
}

template <typename T>
MyArrayList<T>::MyArrayList(const MyArrayList & other){
    m_capacity = other.m_capacity;
    m_size = other.m_size;
    m_elements = new T[m_capacity];
    std::copy(other.m_elements, other.m_elements + m_size, m_elements);
}

template <typename T>
MyArrayList<T>& MyArrayList<T>::operator=(const MyArrayList<T>& other){
    m_capacity = other.m_capacity;
    m_size = other.m_size;
    delete [] m_elements;
    m_elements = new T[m_capacity];
    for(int i=0; i < m_size ; ++i)
        m_elements[i] = other.m_elements[i];
}

template <typename T>
MyArrayList<T>::~MyArrayList() {
    delete[] m_elements;
}

template <typename T>
void MyArrayList<T>::add(T value) {
    checkResize();
    m_elements[m_size] = value;
    m_size++;
}

template <typename T>
void MyArrayList<T>::clear() {
    zeroOut(m_elements, m_capacity);
    m_size = 0;
}

template <typename T>
int MyArrayList<T>::get(int index) const {
    checkIndex(index,0,m_size-1);
    return m_elements[index];
}

template <typename T>
void MyArrayList<T>::insert(int index, T value) {
    checkResize();

    // shift right to make room
    for (int i = m_size; i > index; i--) {
        m_elements[i] = m_elements[i - 1];
    }
    m_elements[index] = value;
    m_size++;
}

template <typename T>
bool MyArrayList<T>::isEmpty() const {
    return m_size == 0;
}

template <typename T>
void MyArrayList<T>::remove(int index) {
    for (int i = index; i < m_size; i++) {
        m_elements[i] = m_elements[i + 1];
    }
    m_size--;
}

template <typename T>
void MyArrayList<T>::set(int index, T value) {
    m_elements[index] = value;
}

template <typename T>
int MyArrayList<T>::size() const {
    return m_size;
}

template <typename T>
string MyArrayList<T>::toString() const {
    string s = "[";
    if (!isEmpty()) {
        s += to_string(m_elements[0]);
        for (int i = 1; i < m_size; i++) {
            s += ", ";
            s += to_string(m_elements[i]);
        }
    }
    s += "]";
    return s;
}

template <typename T>
void MyArrayList<T>::checkResize() {
    if (m_size == m_capacity) {
        // out of space; resize
        T* bigDaddy = new T[m_capacity * 2];
        for (int i = 0; i < m_size; i++) {
            bigDaddy[i] = m_elements[i];
        }
       delete[] m_elements;   // free old array's memory
        m_elements = bigDaddy;
        m_capacity *= 2;
    }
}

template <typename T>
void MyArrayList<T>::checkIndex(int i, int min, int max) const{

    if(i < min || i > max){
        throw std::out_of_range("index " + std::to_string(i) +
                                " out of range (must be between " +
                                std::to_string(min) + " and " +
                                std::to_string(max) + ")");
    }

}

// helper to set all m_elements of an int array to 0.
template <typename T>
static void zeroOut(T* a, int length) {
    for (int i = 0; i < length; i++) {
        a[i] = 0;
    }
}
